--[[
Version: 3.3.0_20091230_1

Originaly created by Global, current author of Perl Classic Units Frame

Perl Player Buff is now maintained by Leliel AKA :
- Leliel at Curse.com
- Leliel, Yui, Nerv, Neon at EU-Ysondre

A set of new features has been added, all off by default and managed by a separeted config frame.

List of new features (all toggleable):
- Set the number of buffs per line
- Display native blizzard cooldown on buff icon
- Hide perl text timer under buffs icon
- Handle weapon temporary enchant
- Display castable buffs only
- Display curable buffs only
- Show seconds under 10 minutes even when "hide seconds" is checked

Planned features:
- Adjustable vertical spacing
- Adjustable x offset
- Adjustable y offset
- Text timer font auto resize to fit available space


--]]

---------------
-- Variables --
---------------
local Perl_Player_Buff_Version = "3.3.0_20091230_1";

Perl_Player_Buff_Config = {};

-- New Enhanced config table 
PPB_Enhanced_Config = {};

-- Hook Perl Config Function
local Original_Perl_Player_Set_Hide_Class_Level_Frame = nil;
local Original_Perl_Player_XPBar_Display = nil


local Perl_Player_Buff_Events = {}; -- event manager

-- Default Saved Variables (also set in Perl_Player_Buff_GetVars)
local buffalerts = 1;       -- alerts are enabled by default
local showbuffs = 1;        -- mod is on be default
local scale = 1;        -- default scale
local hideseconds = 0;      -- seconds are shown by default
local horizontalspacing = 10;   -- default horizontal spacing

-- Default Enhanced Saved Variables (also set in Perl_Player_Buff_GetVars_Enhanced)
local PPBEC_BuffPerLine = 10; -- coded
local PPBEC_ShowNativeCoolDown = false; -- done
local PPBEC_ShowOriginalTextTimer = true; -- done
local PPBEC_HandleWeaponBuff = false; -- partial, config ok, buff icon todo
local PPBEC_DisplayCastableBuffs = false; -- done
local PPBEC_DisplayCurableDebuff = false; -- done
local PPBEC_ShowSecondUnder10m = false; -- done

-- TODO: planned features
local PPBEC_xOffset = 0;
local PPBEC_yOffset = 0;
local PPBEC_VerticalSpacing = 15;   -- default horizontal spacing

-- Possible features, maybe
local defaultFontName, defaultFontHeight, defaultFontFlags = nil;


-- Default Local Variables
local BuffAlphaMax = 1;     -- Max alpha.
local BuffAlphaMin = 0.25;  -- Minimum alpha for the sine modulation.
local BuffCycleSpeed = 0.75;    -- Cycles per second.
local BuffWarnTime = 31;    -- Onset of flashing
local Initialized = nil;    -- waiting to be initialized
local transparency = 1;     -- 0.8 default from perl
local debuffcheck = {       -- table used for debuff stuff below
    [1] = 0,
    [2] = 0,
    [3] = 0,
    [4] = 0,
    [5] = 0,
    [6] = 0,
    [7] = 0,
    [8] = 0,
    [9] = 0,
    [10] = 0,
    [11] = 0,
    [12] = 0,
    [13] = 0,
    [14] = 0,
    [15] = 0,
    [16] = 0,
    [17] = 0,
    [18] = 0,
    [19] = 0,
    [20] = 0,
    [21] = 0,
    [22] = 0,
    [23] = 0,
    [24] = 0,
    [25] = 0,
    [26] = 0,
    [27] = 0,
    [28] = 0,
    [29] = 0,
    [30] = 0,
    [31] = 0,
    [32] = 0,
    [33] = 0,
    [34] = 0,
    [35] = 0,
    [36] = 0,
    [37] = 0,
    [38] = 0,
    [39] = 0,
    [40] = 0,
}
local Perl_Player_Buff_All_Buttons = nil;

local BuffFrameUpdateTime = 0;
local BuffFrameFlashTime = 0;
local BuffFrameFlashState = 1;
local BuffAlphaValue = 1;


----------------------
-- Loading Function --
----------------------
function Perl_Player_Buff_OnLoad()
    -- Events
    this:RegisterEvent("PLAYER_ENTERING_WORLD");
    this:RegisterEvent("VARIABLES_LOADED");

    -- Scripts
    this:SetScript("OnEvent", Perl_Player_Buff_OnEvent);
    this:SetScript("OnUpdate", Perl_Player_Buff_UpdateAll);
end

function Perl_Player_Buff_Buff_OnLoad()
    this:RegisterForClicks("RightButtonUp");
end


---------------------------
-- Event/Update Handlers --
---------------------------
function Perl_Player_Buff_OnEvent()
    local func = Perl_Player_Buff_Events[event];
    if (func) then
        func();
    else
        if (PCUF_SHOW_DEBUG_EVENTS == 1) then
            DEFAULT_CHAT_FRAME:AddMessage("Perl Classic - Player Buff: Report the following event error to the author: "..event);
        end
    end
end

function Perl_Player_Buff_Events:VARIABLES_LOADED()
    Perl_Player_Buff_Initialize();
end
Perl_Player_Buff_Events.PLAYER_ENTERING_WORLD = Perl_Player_Buff_Events.VARIABLES_LOADED;


-------------------------------
-- Loading Settings Function --
-------------------------------
function Perl_Player_Buff_Initialize()
    -- Code to be run after zoning or logging in goes here
    if (Initialized) then
        
        Perl_Player_Buff_All_Buttons = {
            [1] = Perl_Player_Buff1,
            [2] = Perl_Player_Buff2,
            [3] = Perl_Player_Buff3,
            [4] = Perl_Player_Buff4,
            [5] = Perl_Player_Buff5,
            [6] = Perl_Player_Buff6,
            [7] = Perl_Player_Buff7,
            [8] = Perl_Player_Buff8,
            [9] = Perl_Player_Buff9,
            [10] = Perl_Player_Buff10,
            [11] = Perl_Player_Buff11,
            [12] = Perl_Player_Buff12,
            [13] = Perl_Player_Buff13,
            [14] = Perl_Player_Buff14,
            [15] = Perl_Player_Buff15,
            [16] = Perl_Player_Buff16,
            [17] = Perl_Player_Buff17,
            [18] = Perl_Player_Buff18,
            [19] = Perl_Player_Buff19,
            [20] = Perl_Player_Buff20,
            [21] = Perl_Player_Buff21,
            [22] = Perl_Player_Buff22,
            [23] = Perl_Player_Buff23,
            [24] = Perl_Player_Buff24,
            [25] = Perl_Player_Buff25,
            [26] = Perl_Player_Buff26,
            [27] = Perl_Player_Buff27,
            [28] = Perl_Player_Buff28,
            [29] = Perl_Player_Buff29,
            [30] = Perl_Player_Buff30,
            [31] = Perl_Player_Buff31,
            [32] = Perl_Player_Buff32,
            [33] = Perl_Player_Buff33,
            [34] = Perl_Player_Buff34,
            [35] = Perl_Player_Buff35,
            [36] = Perl_Player_Buff36,
            [37] = Perl_Player_Buff37,
            [38] = Perl_Player_Buff38,
            [39] = Perl_Player_Buff39,
            [40] = Perl_Player_Buff40
        }
        
        -- save Default Font info
        local timetext = getglobal(Perl_Player_Buff1:GetName().."DurationText");
        defaultFontName, defaultFontHeight, defaultFontFlags = timetext:GetFont();
        
        Perl_Player_Buff_Set_Scale();
        Perl_Player_Buff_Align();
        
        -- hack Perl Config function
        Original_Perl_Player_Set_Hide_Class_Level_Frame = Perl_Player_Set_Hide_Class_Level_Frame;
        Perl_Player_Set_Hide_Class_Level_Frame = Enhanced_Perl_Player_Set_Hide_Class_Level_Frame;
        
        Original_Perl_Player_XPBar_Display = Perl_Player_XPBar_Display
        Perl_Player_XPBar_Display = Enhanced_Perl_Player_XPBar_Display
        return;
    end

    -- Check if a previous exists, if not, enable by default.
    if (type(Perl_Player_Buff_Config[UnitName("player")]) == "table") then
        Perl_Player_Buff_GetVars();
    else
        Perl_Player_Buff_UpdateVars();
    end

    -- Check if a previous exists, if not, enable by default.
    if (type(PPB_Enhanced_Config[UnitName("player")]) == "table") then
        Perl_Player_Buff_GetVars_Enhanced();
    else
        Perl_Player_Buff_UpdateVars_Enhanced();
    end

    -- Major config options.
    Perl_Player_BuffFrame:SetBackdropColor(0, 0, 0, 1);
    Perl_Player_BuffFrame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1);
    Perl_Player_UseBuffs(showbuffs);

    -- MyAddOns Support
    Perl_Player_Buff_myAddOns_Support();

--[[ Debuff Color Blizz Bug fix
    if false and DebuffTypeColor == nil then
        DebuffTypeColor = { };
        DebuffTypeColor["none"] = { r = 0.80, g = 0, b = 0 };
        DebuffTypeColor["Magic"]    = { r = 0.20, g = 0.60, b = 1.00 };
        DebuffTypeColor["Curse"]    = { r = 0.60, g = 0.00, b = 1.00 };
        DebuffTypeColor["Disease"]  = { r = 0.60, g = 0.40, b = 0 };
        DebuffTypeColor["Poison"]   = { r = 0.00, g = 0.60, b = 0 };
        DebuffTypeColor[""] = DebuffTypeColor["none"];
        DEFAULT_CHAT_FRAME:AddMessage("|cff00ffff Load DebuffTypeColor upon blizz one";
    end
--]]

    Initialized = 1;
end


--------------------
-- Buff Functions --
--------------------
function Perl_Player_Buff_UpdateAlpha(self, elapsed)
    if ( BuffFrameUpdateTime > 0 ) then
       BuffFrameUpdateTime = BuffFrameUpdateTime - elapsed;
    else
        BuffFrameUpdateTime = BuffFrameUpdateTime + TOOLTIP_UPDATE_TIME;
    end
     
    BuffFrameFlashTime = BuffFrameFlashTime - elapsed;
    if ( BuffFrameFlashTime < 0 ) then
        local overtime = -BuffFrameFlashTime;
        if (BuffFrameFlashState == 0 ) then
            BuffFrameFlashState = 1;
            BuffFrameFlashTime = BUFF_FLASH_TIME_ON;
        else
            BuffFrameFlashState = 0;
            BuffFrameFlashTime = BUFF_FLASH_TIME_OFF;
        end
        if ( overtime < BuffFrameFlashTime ) then
            BuffFrameFlashTime = BuffFrameFlashTime - overtime;
        end
    end
     
    if ( BuffFrameFlashState == 1 ) then
        BuffAlphaValue = (BUFF_FLASH_TIME_ON - BuffFrameFlashTime) / BUFF_FLASH_TIME_ON;
    else
        BuffAlphaValue = BuffFrameFlashTime / BUFF_FLASH_TIME_ON;
    end
        BuffAlphaValue = (BuffAlphaValue * (1 - BUFF_MIN_ALPHA)) + BUFF_MIN_ALPHA;
end

function Perl_Player_Buff_UpdateAll(self, elapsed)
    
    Perl_Player_Buff_UpdateAlpha(self, elapsed);
    
    if (showbuffs == 1) then
        local totalBuff = 0;
        local buffnumOffset = 0;
        
        -- Weapon Temporary Enchant Management
        if PPBEC_HandleWeaponBuff == true then
            local hasMainHandEnchant, mainHandExpiration, mainHandCharges, hasOffHandEnchant, offHandExpiration, offHandCharges = GetWeaponEnchantInfo();
            
            if ( hasMainHandEnchant ) then
                buffnumOffset = buffnumOffset + 1;
                local textureName;
                local button = getglobal("Perl_Player_Buff"..(buffnumOffset));
                buffName = "MainHandEnchant";
                textureName = GetInventoryItemTexture("player", 16);
                
                -- local debuff = getglobal(button:GetName().."DebuffBorder");
                local timetext = getglobal(button:GetName().."DurationText");
                local debuffCount = getglobal(button:GetName().."Count");
                getglobal(button:GetName().."Icon"):SetTexture(textureName);
                
                -- reset to default
                timetext:SetFont(defaultFontName, defaultFontHeight, defaultFontFlags);
                
                if (mainHandCharges > 1) then
                    debuffCount:SetText(mainHandCharges);
                    debuffCount:Show();
                else
                    debuffCount:Hide();
                end
                
                -- debuff:Hide();              -- Set this to hidden so the debuff loop below can properly set it
                timetext:SetTextColor(1, 1, 0, 1);  -- Set this to default color so the debuff loop below can properly set it
                
                if ( mainHandExpiration ) then
                    mainHandExpiration = mainHandExpiration/1000;
                end
                local buffETA = mainHandExpiration;
                
                -- text timer
                if (PPBEC_ShowOriginalTextTimer == true and buffETA >= 0) then
                    timetext:SetText(Perl_Player_GetStringTime(buffETA));
                    timetext:Show();
                else
                    timetext:Hide();
                end
                
                -- Alpha effect and warning
                if (buffETA >= 0) then
                    if (buffETA < BuffWarnTime) then
                        button:SetAlpha(BuffAlphaValue);
                        if ((button.isdebuff == 0) and (button.notwarned == 1) and (buffalerts == 1)) then
                            if (GetLocale() == "zhCN") then
                                UIErrorsFrame:AddMessage(buffname.." 将在30秒后消失.", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<Main Hand Enchant>|cffffff00 将在30秒后消失.");
                            else
                                UIErrorsFrame:AddMessage(buffname.." will expire in 30 seconds", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<Main Hand Enchant>|cffffff00 will expire in 30 seconds");
                            end
                            button.notwarned = 0;
                        else
                            button.notwarned = 0;
                        end
                    else
                        button:SetAlpha(1);
                        if (button.notwarned ~= 1) then
                            button.notwarned = 1;
                        end
                    end
                else
                    button:SetAlpha(1);
                    button.notwarned = 0;
                end
                
                -- cooldown
                cooldown = getglobal(button:GetName().."Cooldown");
                if (PPBEC_ShowNativeCoolDown and PPBEC_ShowNativeCoolDown == true) then
                    local startTime, duration= (GetTime()-60*60+mainHandExpiration), 60*60;
                    if duration and duration > 0 then
                        CooldownFrame_SetTimer(cooldown, startTime, duration, 1);
                        cooldown:Show();
                    else
                        CooldownFrame_SetTimer(cooldown, 0, 0, 0);
                        cooldown:Hide();
                    end
                else
                    CooldownFrame_SetTimer(cooldown, 0, 0, 0);
                    cooldown:Hide();
                end
                
                if button.name ~= buffName then
                    button.name = buffName;
                    button.notwarned = 1;
                end
                button:Show();
            end
            
            if ( hasOffHandEnchant ) then
                buffnumOffset = buffnumOffset + 1;
                local textureName;
                local button = getglobal("Perl_Player_Buff"..(buffnumOffset));
                buffName = "OffHandEnchant";
                textureName = GetInventoryItemTexture("player", 17);
                
                -- local debuff = getglobal(button:GetName().."DebuffBorder");
                local timetext = getglobal(button:GetName().."DurationText");
                local debuffCount = getglobal(button:GetName().."Count");
                getglobal(button:GetName().."Icon"):SetTexture(textureName);
                
                -- reset to default
                timetext:SetFont(defaultFontName, defaultFontHeight, defaultFontFlags);
                
                if (offHandCharges > 1) then
                    debuffCount:SetText(offHandCharges);
                    debuffCount:Show();
                else
                    debuffCount:Hide();
                end
                
                -- debuff:Hide();              -- Set this to hidden so the debuff loop below can properly set it
                timetext:SetTextColor(1, 1, 0, 1);  -- Set this to default color so the debuff loop below can properly set it
                
                if ( offHandExpiration ) then
                    offHandExpiration = offHandExpiration/1000;
                end
                local buffETA = offHandExpiration;
                
                -- text timer
                if (PPBEC_ShowOriginalTextTimer == true and buffETA >= 0) then
                    timetext:SetText(Perl_Player_GetStringTime(buffETA));
                    timetext:Show();
                else
                    timetext:Hide();
                end
                
                -- Alpha effect and warning
                if (buffETA >= 0) then
                    if (buffETA < BuffWarnTime) then
                        button:SetAlpha(BuffAlphaValue);
                        if ((button.isdebuff == 0) and (button.notwarned == 1) and (buffalerts == 1)) then
                            if (GetLocale() == "zhCN") then
                                UIErrorsFrame:AddMessage(buffname.." 将在30秒后消失.", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<Off Hand Enchant>|cffffff00 将在30秒后消失.");
                            else
                                UIErrorsFrame:AddMessage(buffname.." will expire in 30 seconds", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<Off Hand Enchant>|cffffff00 will expire in 30 seconds");
                            end
                            button.notwarned = 0;
                        else
                            button.notwarned = 0;
                        end
                    else
                        button:SetAlpha(1);
                        if (button.notwarned ~= 1) then
                            button.notwarned = 1;
                        end
                    end
                else
                    button:SetAlpha(1);
                    button.notwarned = 0;
                end
                
                -- cooldown
                cooldown = getglobal(button:GetName().."Cooldown");
                if (PPBEC_ShowNativeCoolDown and PPBEC_ShowNativeCoolDown == true) then
                    local startTime, duration= (GetTime()-60*60+offHandExpiration), 60*60;
                    if duration and duration > 0 then
                        CooldownFrame_SetTimer(cooldown, startTime, duration, 1);
                        cooldown:Show();
                    else
                        CooldownFrame_SetTimer(cooldown, 0, 0, 0);
                        cooldown:Hide();
                    end
                else
                    CooldownFrame_SetTimer(cooldown, 0, 0, 0);
                    cooldown:Hide();
                end
                
                if button.name ~= buffName then
                    button.name = buffName;
                    button.notwarned = 1;
                end
                button:Show();
            end
        end
        
        -- BUFF LOOP
        for buffnum=1,40-buffnumOffset do
            local button = getglobal("Perl_Player_Buff"..(buffnum+buffnumOffset));
            
            if (PPBEC_DisplayCastableBuffs == true) then
                bufffilter = "HELPFUL PLAYER";
            else
                bufffilter = "HELPFUL";
            end
            --        name,     rank,     icon,              count, debuffType, duration,     expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitAura("unit", index or ["name", "rank"][, "filter"])
            local buffName, buffRank, buffIcon, debuffApplications, debuffType, duration, buffExpirationTime, unitCaster, isStealable                              = UnitAura("player", buffnum, bufffilter)
            
            if (buffName ~= nil) then
                totalBuff = totalBuff + 1;
                
                local debuff = getglobal(button:GetName().."DebuffBorder");
                local timetext = getglobal(button:GetName().."DurationText");
                local debuffCount = getglobal(button:GetName().."Count");
                getglobal(button:GetName().."Icon"):SetTexture(buffIcon);
                
                -- reset to default
                timetext:SetFont(defaultFontName, defaultFontHeight, defaultFontFlags);
                
                if (debuffApplications > 1) then
                    debuffCount:SetText(debuffApplications);
                    debuffCount:Show();
                else
                    debuffCount:Hide();
                end
                
                debuff:Hide();              -- Set this to hidden so the debuff loop below can properly set it
                timetext:SetTextColor(1, 1, 0, 1);  -- Set this to default color so the debuff loop below can properly set it
                
                if (PPBEC_ShowOriginalTextTimer == true and buffExpirationTime ~= 0) then
                    local buffETA = buffExpirationTime-GetTime();
                    timetext:SetText(Perl_Player_GetStringTime(buffETA));
                    timetext:Show();
                    
                    if (buffETA < BuffWarnTime) then
                        button:SetAlpha(BuffAlphaValue);

                        if ((button.isdebuff == 0) and (button.notwarned == 1) and (button.name == buffname) and (buffalerts == 1)) then
                            if (GetLocale() == "zhCN") then
                                UIErrorsFrame:AddMessage(buffname.." 将在30秒后消失.", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<"..buffname..">|cffffff00 将在30秒后消失.");
                            else
                                UIErrorsFrame:AddMessage(buffname.." will expire in 30 seconds", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<"..buffname..">|cffffff00 will expire in 30 seconds");
                            end
                            button.notwarned = 0;
                        else
                            button.notwarned = 0;
                        end
                    else
                        button:SetAlpha(1);
                        if (button.notwarned ~= 1) then
                            button.notwarned = 1;
                        end
                    end
                else
                    timetext:Hide();
                    button:SetAlpha(1);
                    button.notwarned = 0;
                end
                
                cooldown = getglobal(button:GetName().."Cooldown");
                if (PPBEC_ShowNativeCoolDown and PPBEC_ShowNativeCoolDown == true and duration and duration > 0) then
                    CooldownFrame_SetTimer(cooldown, buffExpirationTime - duration, duration, 1);
                    cooldown:Show();
                else
                    CooldownFrame_SetTimer(cooldown, 0, 0, 0);
                    cooldown:Hide();
                end

                if button.name ~= buffName then
                    button.name = buffName;
                    button.notwarned = 0;
                end
                button:Show();
            else
                button:Hide();
                button.notwarned = 0;
            end
        end
        
        -- DEBUFF LOOP
        for buffnum=totalBuff+1,40-buffnumOffset do
            local button = getglobal("Perl_Player_Buff"..(buffnum+buffnumOffset));
            
            if (PPBEC_DisplayCurableDebuff == true) then                               -- Are we targeting a friend or enemy and which filter do we need to apply?
                debuffFilter = "HARMFUL RAID";
            else
                debuffFilter = "HARMFUL";
            end
--                    name,     rank,     icon,              count, debuffType, duration,     expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitAura("unit", index or ["name", "rank"][, "filter"]) 
            local buffName, buffRank, buffIcon, debuffApplications, debuffType, duration, buffExpirationTime, unitCaster, isStealable                              = UnitAura("player", buffnum-totalBuff, debuffFilter)
            
            local isValidDebuff = true;
            if (PPBEC_DisplayCurableDebuff == true) then
                isValidDebuff = (Perl_Config_Set_Curable_Debuffs(debuffType) == 1);
            end
            
            if (buffName ~= nil and isValidDebuff) then

                local debuff = getglobal(button:GetName().."DebuffBorder");
                local timetext = getglobal(button:GetName().."DurationText");
                local debuffCount = getglobal(button:GetName().."Count");
                getglobal(button:GetName().."Icon"):SetTexture(buffIcon);
                
                if (debuffApplications > 1) then
                    debuffCount:SetText(debuffApplications);
                    debuffCount:Show();
                else
                    debuffCount:Hide();
                end
                
                debuffcheck[buffnum] = 1;
                button.isdebuff = 1;

                debuff:Hide();              -- Set this to hidden so the debuff loop below can properly set it
                timetext:SetTextColor(1, 1, 0, 1);  -- Set this to default color so the debuff loop below can properly set it

                if (PPBEC_ShowOriginalTextTimer and PPBEC_ShowOriginalTextTimer == true and buffExpirationTime ~= 0) then
                    local buffETA = buffExpirationTime-GetTime();
                    timetext:SetText(Perl_Player_GetStringTime(buffETA));
                    
                    timetext:Show();

                    if (buffETA < BuffWarnTime) then
                        button:SetAlpha(BuffAlphaValue);

                        if ((button.isdebuff == 0) and (button.notwarned == 1) and (button.name == buffname) and (buffalerts == 1)) then
                            if (GetLocale() == "zhCN") then
                                UIErrorsFrame:AddMessage(buffname.." 将在30秒后消失.", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<"..buffname..">|cffffff00 将在30秒后消失.");
                            else
                                UIErrorsFrame:AddMessage(buffname.." will expire in 30 seconds", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                                DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<"..buffname..">|cffffff00 will expire in 30 seconds");
                            end
                            button.notwarned = 0;
                        else
                            button.notwarned = 0;
                        end
                    else
                        button:SetAlpha(1);
                        if (button.notwarned ~= 1) then
                            button.notwarned = 1;
                        end
                    end
                else
                    timetext:Hide();
                    button:SetAlpha(1);
                    button.notwarned = 0;
                end
                
                cooldown = getglobal(button:GetName().."Cooldown");
                if (PPBEC_ShowNativeCoolDown and PPBEC_ShowNativeCoolDown == true and duration and duration > 0) then
                    CooldownFrame_SetTimer(cooldown, buffExpirationTime - duration, duration, 1);
                    cooldown:Show();
                else
                    CooldownFrame_SetTimer(cooldown, 0, 0, 0);
                    cooldown:Hide();
                end

                if button.name ~= buffName then
                    button.name = buffName;
                    button.notwarned = 0;
                end
                button:Show();
            else
                button:Hide();
                button.notwarned = 0;
            end
        end

        if buffnumOffset > 0 then
            local button;
            local border;
            button = getglobal("Perl_Player_Buff1");
            if button.name == "MainHandEnchant" then
                border = getglobal("Perl_Player_Buff1DebuffBorder");
                local r, g, b, hex = GetItemQualityColor(GetInventoryItemQuality("player", 16));
                border:SetVertexColor(r, g, b);
                border:Show();
            elseif button.name == "OffHandEnchant" then
                border = getglobal("Perl_Player_Buff1DebuffBorder");
                local r, g, b, hex = GetItemQualityColor(GetInventoryItemQuality("player", 17));
                border:SetVertexColor(r, g, b);
                border:Show();
            end
            
            button = getglobal("Perl_Player_Buff2");
            if button.name == "MainHandEnchant" then
                border = getglobal("Perl_Player_Buff2DebuffBorder");
                local r, g, b, hex = GetItemQualityColor(GetInventoryItemQuality("player", 16));
                border:SetVertexColor(r, g, b);
                border:Show();
            elseif button.name == "OffHandEnchant" then
                border = getglobal("Perl_Player_Buff2DebuffBorder");
                local r, g, b, hex = GetItemQualityColor(GetInventoryItemQuality("player", 17));
                border:SetVertexColor(r, g, b);
                border:Show();
            end
        end
        
        local debufftotal = 0;
        for buffnum=totalBuff+1,40 do           -- Read the debuff table and set debuffs accordingly
            if (debuffcheck[buffnum] == 1) then
                debufftotal = debufftotal + 1;
                local debuff = getglobal("Perl_Player_Buff"..buffnum.."DebuffBorder");
--                    name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId  = UnitAura("unit", index or ["name", "rank"][, "filter"]) 
                local _   , _   , _   , _    , debuffType                                                                                 = UnitAura("player", buffnum-totalBuff, debuffFilter);
                if (debuffType) then
                    color = DebuffTypeColor[debuffType];
                else
                    color = DebuffTypeColor[PERL_LOCALIZED_BUFF_NONE];
                end
                debuff:SetVertexColor(color.r, color.g, color.b);
                debuff:Show();
                if PPBEC_ShowOriginalTextTimer == 1 then
                    local timetext = getglobal("Perl_Player_Buff"..buffnum.."DurationText");
                    timetext:SetTextColor(color.r, color.g, color.b, 1);
                end
            end
            debuffcheck[buffnum] = 0;   -- Reset the debuff table
        end
        Perl_Player_BuffFrame:Show();
    else
        Perl_Player_BuffFrame:Hide();
    end
end


function Perl_Player_GetStringTime(timenum)
    -- little fix of time drift
    timenum = floor(timenum + 0.5);
    
    local days = math.floor(timenum / (24*60*60));
    local hours = math.floor(timenum / (60*60) - (days*60*60));
    local minutes = math.floor(timenum / 60 - (days*60*60) - (hours*60));
    local seconds = math.floor(timenum - (days*60*60) - (hours*60) - (minutes*60));
    
    local shours, sminutes, sseconds, smili, timestring;
    
    if seconds < 0 then
        seconds = 0
    end
    
    if (hours>0 and string.len(hours) == 1) then
        shours = "0"..hours;
    else
        shours = ""..hours;
    end
    if (minutes>0 and string.len(minutes) == 1) then
        sminutes = "0"..minutes;
    else
        sminutes = ""..minutes;
    end
    if (string.len(seconds) == 1) then
        sseconds = "0"..seconds;
    else
        sseconds = ""..seconds;
    end
    
    if (days > 0) then
        if (hideseconds == 1) then
            timestring = days.."d"..shours.."h"..sminutes.."m";
        else
            timestring = days..":"..shours..":"..sminutes..":"..sseconds;
        end
    elseif (hours > 0) then
        if (hideseconds == 1) then
            timestring = shours.."h "..sminutes.."m";
        else
            timestring = shours..":"..sminutes..":"..sseconds;
        end
    elseif (minutes > 0) then
        if (hideseconds == 1) then
            if (PPBEC_ShowSecondUnder10m and PPBEC_ShowSecondUnder10m == true and minutes < 10) then
                timestring = sminutes..":"..sseconds; -- <9:59
            else
                timestring = sminutes.."m"; -- >10m
            end
        else
            timestring = sminutes..":"..sseconds;
        end
    else -- less than a minute
        timestring = sseconds.."s";
    end
    return timestring;
end

function Perl_Player_SetBuffTooltip()
    for buffnum=1,40 do
        local buffName, buffRank = nil;
        buffName, buffRank = UnitAura("player", buffnum, "HELPFUL");
        if buffName ~= nil and buffName == this.name then
            GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
            GameTooltip:SetUnitAura("player", buffName, buffRank);
            return;
        elseif this.name == "MainHandEnchant" then
            GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
            GameTooltip:SetInventoryItem("player", 16);
            return;
        elseif this.name == "OffHandEnchant" then
            GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
            GameTooltip:SetInventoryItem("player", 17);
            return;
        end
        
        buffName = UnitAura("player", buffnum, "HARMFUL");
        if buffName ~= nil and buffName == this.name then
           GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
           GameTooltip:SetUnitDebuff("player", buffName);
           return;
        end
    end
end

function Perl_Player_BuffClicked(button)
    if (button == "RightButton") then
        if this.name == "MainHandEnchant" then
            CancelItemTempEnchantment(1);
            this.notwarned = 0;
        elseif this.name == "OffHandEnchant" then
            CancelItemTempEnchantment(2);
            this.notwarned = 0;
        else
            for buffnum=1,40 do
                local buffName, buffRank = UnitAura("player", buffnum, "HELPFUL");
                if buffName ~= nil and buffName == this.name then
                    CancelUnitBuff("player", buffName, buffRank);
                    this.notwarned = 0;
                    break;
                end
            end
        end
    end
end

function Perl_Player_UseBuffs(useperlbuffs)
    if (useperlbuffs == 1) then
        BuffFrame:Hide();
        if PPBEC_HandleWeaponBuff == true then
            TemporaryEnchantFrame:Hide()
        else
            TemporaryEnchantFrame:Show();
        end
    else
        BuffFrame:Show();
        TemporaryEnchantFrame:Show();
    end
end

-- Compatibility wrapper
function Perl_Player_Buff_Allign()
    --DEFAULT_CHAT_FRAME:AddMessage("|cff0000ff Perl use a deprecated function: Perl_Player_Buff_Allign()");
    Perl_Player_Buff_Align()
end

function Perl_Player_Buff_Align()
    local name = nil;
    local hideclasslevelframe = 0;
    
    local xFirstOffset = PPBEC_xOffset;
    local yFirstOffset = PPBEC_yOffset;
    local buttonPerLine = PPBEC_BuffPerLine;
    
    name = UnitName("player");
    
    if (type(Perl_Player_Config[name]) == "table") then
        if Perl_Player_Config[name]["HideClassLevelFrame"] then
            hideclasslevelframe = Perl_Player_Config[name]["HideClassLevelFrame"];
        end 
    end
        
    local TopLeftAnchorFrame  = nil; -- the first frame of current line, needed to anchor the first of the next line
    local CurrentAnchorFrame  = nil; -- the last frame anchored, needed to anchor the next
    
    -- init first anchor relatively to the most left displayed frame
    Perl_Player_Buff1:ClearAllPoints();
    if hideclasslevelframe == 1 then
        Perl_Player_Buff1:SetPoint("TOPLEFT", Perl_Player_StatsFrame, "BOTTOMLEFT", xFirstOffset, yFirstOffset);
    else
        if Perl_Player_XPBar and Perl_Player_XPBar:IsVisible() then
            yFirstOffset = yFirstOffset-(Perl_Player_XPBar:GetHeight());
        end
        Perl_Player_Buff1:SetPoint("TOPLEFT", Perl_Player_LevelFrame, "BOTTOMLEFT", xFirstOffset, yFirstOffset);
    end
    
    -- init TopLeftAnchorFrame
    TopLeftAnchorFrame = 1;
    CurrentAnchorFrame = 1;
    
    -- let's go !
    for buffButtonIndex=2,40 do
        -- if buffButtonIndex match the first index of a line, Anchor to TopLeftAnchorFrame
        if ((buffButtonIndex-1)%buttonPerLine) == 0 then
            
            -- new line, Set Current to topleft
            CurrentAnchorFrame = TopLeftAnchorFrame;
            
            -- and Set next Topleft to current one
            TopLeftAnchorFrame = buffButtonIndex;
            
            -- Set Next Button Anchor and Position, always on CurrentAnchorFrame
            Perl_Player_Buff_All_Buttons[buffButtonIndex]:ClearAllPoints();
            Perl_Player_Buff_All_Buttons[buffButtonIndex]:SetPoint("TOPLEFT", Perl_Player_Buff_All_Buttons[CurrentAnchorFrame], "BOTTOMLEFT", 0, -PPBEC_VerticalSpacing);
        else
            -- Set Next Button Anchor and Position, always on CurrentAnchorFrame
            Perl_Player_Buff_All_Buttons[buffButtonIndex]:ClearAllPoints();
            Perl_Player_Buff_All_Buttons[buffButtonIndex]:SetPoint("TOPLEFT", Perl_Player_Buff_All_Buttons[CurrentAnchorFrame], "TOPRIGHT", horizontalspacing, 0);
        end
        
        -- Set current for next :)
        CurrentAnchorFrame = buffButtonIndex;
    end
end


--------------------------
-- GUI Config Functions --
--------------------------
function Perl_Player_Buff_Set_ShowBuffs(newvalue)
    showbuffs = newvalue;
    Perl_Player_Buff_UpdateVars();
    Perl_Player_UseBuffs(showbuffs);
end

function Perl_Player_Buff_Set_Alerts(newvalue)
    buffalerts = newvalue;
    Perl_Player_Buff_UpdateVars();
end

function Perl_Player_Buff_Set_Hide_Seconds(newvalue)
    hideseconds = newvalue;
    Perl_Player_Buff_UpdateVars();
end

function Perl_Player_Buff_Set_Horizontal_Spacing(number)
    if (number ~= nil) then
        horizontalspacing = number;
    else
        horizontalspacing = 10;
    end
    Perl_Player_Buff_UpdateVars();
    Perl_Player_Buff_Align();
end

function Perl_Player_Buff_Set_Scale(number)
    local unsavedscale;
    if (number ~= nil) then
        scale = (number / 100);                 -- convert the user input to a wow acceptable value
    else
        scale = 1;
    end
    unsavedscale = 1 - UIParent:GetEffectiveScale() + scale;    -- run it through the scaling formula introduced in 1.9
    Perl_Player_BuffFrame:SetScale(unsavedscale);
    Perl_Player_Buff_UpdateVars();
end

-----------------------------------
-- GUI Enhanced Config Functions --
-----------------------------------

function Perl_Player_Buff_Set_BuffPerLine(number)
    if (number ~= nil) then
        PPBEC_BuffPerLine = number;
    else
        PPBEC_BuffPerLine = 10;
    end
    Perl_Player_Buff_UpdateVars_Enhanced();
    Perl_Player_Buff_Align();
end

function Perl_Player_Buff_Set_ShowNativeCoolDown(newvalue)
    PPBEC_ShowNativeCoolDown = newvalue;
    Perl_Player_Buff_UpdateVars_Enhanced();
end

local lastHS, lastVS = 10, 15;
function Perl_Player_Buff_Set_ShowOriginalTextTimer(newvalue)
    PPBEC_ShowOriginalTextTimer = newvalue;
    
    if PPBEC_ShowOriginalTextTimer == false then
        --lastVS = PPBEC_VerticalSpacing;
        lastHS = horizontalspacing;
        PPBEC_VerticalSpacing = 1;
        horizontalspacing = 1;
    else
        if horizontalspacing == 1 then
            horizontalspacing = lastHS;
        end
        PPBEC_VerticalSpacing = 15;
    end
    Perl_Player_Buff_UpdateVars_Enhanced();
    Perl_Player_Buff_Align();
end

function Perl_Player_Buff_Set_HandleWeaponBuff(newvalue)
    PPBEC_HandleWeaponBuff = newvalue;
    Perl_Player_Buff_UpdateVars_Enhanced();
    Perl_Player_UseBuffs(showbuffs);
end

function Perl_Player_Buff_Set_DisplayCastableBuffs(newvalue)
    PPBEC_DisplayCastableBuffs = newvalue;
    Perl_Player_Buff_UpdateVars_Enhanced();
end

function Perl_Player_Buff_Set_DisplayCurableDebuff(newvalue)
    PPBEC_DisplayCurableDebuff = newvalue;
    Perl_Player_Buff_UpdateVars_Enhanced();
end

function Perl_Player_Buff_Set_ShowSecondUnder10m(newvalue)
    PPBEC_ShowSecondUnder10m = newvalue;
    Perl_Player_Buff_UpdateVars_Enhanced();
end

-- TODO 
function Perl_Player_Buff_Set_Vertical_Spacing(number)
    if (number ~= nil) then
        PPBEC_VerticalSpacing = number;
    else
        PPBEC_VerticalSpacing = 15;
    end
    Perl_Player_Buff_UpdateVars_Enhanced();
    Perl_Player_Buff_Align();
end

-- Hook Player Config Function for hideclasslevelframe Update
function Enhanced_Perl_Player_Set_Hide_Class_Level_Frame(newvalue)
    -- DEFAULT_CHAT_FRAME:AddMessage("|cff00ffff Enhanced_Perl_Player_Set_Hide_Class_Level_Frame");
    Original_Perl_Player_Set_Hide_Class_Level_Frame(newvalue);
    Perl_Player_Buff_Align();
end

-- Hook Player Config Function for xpbarstate Update
function Enhanced_Perl_Player_XPBar_Display(newvalue)
    Original_Perl_Player_XPBar_Display(newvalue);
    Perl_Player_Buff_Align();
end

------------------------------
-- Saved Variable Functions --
------------------------------
function Perl_Player_Buff_GetVars(name, updateflag)
    if (name == nil) then
        name = UnitName("player");
    end

    buffalerts = Perl_Player_Buff_Config[name]["buffalerts"];
    showbuffs = Perl_Player_Buff_Config[name]["showbuffs"];
    scale = Perl_Player_Buff_Config[name]["scale"];
    hideseconds = Perl_Player_Buff_Config[name]["hideseconds"];
    horizontalspacing = Perl_Player_Buff_Config[name]["horizontalspacing"];
    
    if (buffalerts == nil) then
        buffalerts = 1;
    end
    if (showbuffs == nil) then
        showbuffs = 1;
    end
    if (scale == nil) then
        scale = 1;
    end
    if (hideseconds == nil) then
        hideseconds = 0;
    end
    if (horizontalspacing == nil) then
        horizontalspacing = 10;
    end
    
    if (updateflag == 1) then
        -- Save the new values
        Perl_Player_Buff_UpdateVars();
        
        -- Call any code we need to activate them
        Perl_Player_UseBuffs(showbuffs);
        Perl_Player_Buff_Set_Scale();
        return;
    end

    local vars = {
        ["buffalerts"] = buffalerts,
        ["showbuffs"] = showbuffs,
        ["scale"] = scale,
        ["hideseconds"] = hideseconds,
        ["horizontalspacing"] = horizontalspacing,
    }
    return vars;
end

function Perl_Player_Buff_UpdateVars(vartable)
    if (vartable ~= nil) then
        -- Sanity checks in case you use a load from an old version
        if (vartable["Global Settings"] ~= nil) then
            if (vartable["Global Settings"]["BuffAlerts"] ~= nil) then
                buffalerts = vartable["Global Settings"]["BuffAlerts"];
            else
                buffalerts = nil;
            end
            if (vartable["Global Settings"]["ShowBuffs"] ~= nil) then
                showbuffs = vartable["Global Settings"]["ShowBuffs"];
            else
                showbuffs = nil;
            end
            if (vartable["Global Settings"]["Scale"] ~= nil) then
                scale = vartable["Global Settings"]["Scale"];
            else
                scale = nil;
            end
            if (vartable["Global Settings"]["HideSeconds"] ~= nil) then
                hideseconds = vartable["Global Settings"]["HideSeconds"];
            else
                hideseconds = nil;
            end
            if (vartable["Global Settings"]["HorizontalSpacing"] ~= nil) then
                horizontalspacing = vartable["Global Settings"]["HorizontalSpacing"];
            else
                horizontalspacing = nil;
            end
        end

        -- Set the new values if any new values were found, same defaults as above
        if (buffalerts == nil) then
            buffalerts = 1;
        end
        if (showbuffs == nil) then
            showbuffs = 1;
        end
        if (scale == nil) then
            scale = 1;
        end
        if (hideseconds == nil) then
            hideseconds = 0;
        end
        if (horizontalspacing == nil) then
            horizontalspacing = 10;
        end
        
        -- Call any code we need to activate them
        Perl_Player_UseBuffs(showbuffs);
        Perl_Player_Buff_Set_Scale();
    end

    Perl_Player_Buff_Config[UnitName("player")] = {
        ["buffalerts"] = buffalerts,
        ["showbuffs"] = showbuffs,
        ["scale"] = scale,
        ["hideseconds"] = hideseconds,
        ["horizontalspacing"] = horizontalspacing,
    };
end

-- Enhanced Config Functions duplicate
function Perl_Player_Buff_GetVars_Enhanced(name, updateflag)
    if (name == nil) then
        name = UnitName("player");
    end
    
    PPBEC_BuffPerLine = PPB_Enhanced_Config[name]["BuffPerLine"];
    PPBEC_ShowNativeCoolDown = PPB_Enhanced_Config[name]["ShowNativeCoolDown"];
    PPBEC_ShowOriginalTextTimer = PPB_Enhanced_Config[name]["ShowOriginalTextTimer"];
    PPBEC_HandleWeaponBuff = PPB_Enhanced_Config[name]["HandleWeaponBuff"];
    PPBEC_DisplayCastableBuffs = PPB_Enhanced_Config[name]["DisplayCastableBuffs"];
    PPBEC_DisplayCurableDebuff = PPB_Enhanced_Config[name]["DisplayCurableDebuff"];
    PPBEC_ShowSecondUnder10m = PPB_Enhanced_Config[name]["ShowSecondUnder10m"];
    
    if (PPBEC_BuffPerLine == nil) then
        PPBEC_BuffPerLine = 10;
    end
    if (PPBEC_ShowNativeCoolDown == nil) then
        PPBEC_ShowNativeCoolDown = false;
    end
    if (PPBEC_ShowOriginalTextTimer == nil) then
        PPBEC_ShowOriginalTextTimer = 1;
    end
    if (PPBEC_HandleWeaponBuff == nil) then
        PPBEC_HandleWeaponBuff = false;
    end
    if (PPBEC_DisplayCastableBuffs == nil) then
        PPBEC_DisplayCastableBuffs = false;
    end
    if (PPBEC_DisplayCurableDebuff == nil) then
        PPBEC_DisplayCurableDebuff = false;
    end
    if (PPBEC_ShowSecondUnder10m == nil) then
        PPBEC_ShowSecondUnder10m = false;
    end
    
    if (updateflag == 1) then
        -- Save the new values
        Perl_Player_Buff_UpdateVars_Enhanced();
        
        -- Call any code we need to activate them
        Perl_Player_UseBuffs(showbuffs);
        Perl_Player_Buff_Align(); -- Set new BuffPerLine settings
        return;
    end

    local vars = {
        ["BuffPerLine"] = PPBEC_BuffPerLine,
        ["ShowNativeCoolDown"] = PPBEC_ShowNativeCoolDown,
        ["ShowOriginalTextTimer"] = PPBEC_ShowOriginalTextTimer,
        ["HandleWeaponBuff"] = PPBEC_HandleWeaponBuff,
        ["DisplayCastableBuffs"] = PPBEC_DisplayCastableBuffs,
        ["DisplayCurableDebuff"] = PPBEC_DisplayCurableDebuff,
        ["ShowSecondUnder10m"] = PPBEC_ShowSecondUnder10m,
    }
    return vars;
end

function Perl_Player_Buff_UpdateVars_Enhanced(vartable)
    if (vartable ~= nil) then
        -- Sanity checks in case you use a load from an old version
        if (vartable["Global Settings"] ~= nil) then
            -- Enhanced Config
            if (vartable["Global Settings"]["PPBEC_BuffPerLine"] ~= nil) then
                PPBEC_BuffPerLine = vartable["Global Settings"]["PPBEC_BuffPerLine"];
            else
                PPBEC_BuffPerLine = nil;
            end
            if (vartable["Global Settings"]["PPBEC_ShowNativeCoolDown"] ~= nil) then
                PPBEC_ShowNativeCoolDown = vartable["Global Settings"]["PPBEC_ShowNativeCoolDown"];
            else
                PPBEC_ShowNativeCoolDown = nil;
            end
            if (vartable["Global Settings"]["PPBEC_ShowOriginalTextTimer"] ~= nil) then
                PPBEC_ShowOriginalTextTimer = vartable["Global Settings"]["PPBEC_ShowOriginalTextTimer"];
            else
                PPBEC_ShowOriginalTextTimer = nil;
            end
            if (vartable["Global Settings"]["PPBEC_HandleWeaponBuff"] ~= nil) then
                PPBEC_HandleWeaponBuff = vartable["Global Settings"]["PPBEC_HandleWeaponBuff"];
            else
                PPBEC_HandleWeaponBuff = nil;
            end
            if (vartable["Global Settings"]["PPBEC_DisplayCastableBuffs"] ~= nil) then
                PPBEC_DisplayCastableBuffs = vartable["Global Settings"]["PPBEC_DisplayCastableBuffs"];
            else
                PPBEC_DisplayCastableBuffs = nil;
            end
            if (vartable["Global Settings"]["PPBEC_DisplayCurableDebuff"] ~= nil) then
                PPBEC_DisplayCurableDebuff = vartable["Global Settings"]["PPBEC_DisplayCurableDebuff"];
            else
                PPBEC_DisplayCurableDebuff = nil;
            end
            if (vartable["Global Settings"]["ShowSecondUnder10m"] ~= nil) then
                ShowSecondUnder10m = vartable["Global Settings"]["ShowSecondUnder10m"];
            else
                ShowSecondUnder10m = nil;
            end
        end
        
        -- Set the new values for Enhanced config
        if (PPBEC_BuffPerLine == nil) then
            PPBEC_BuffPerLine = 10;
        end
        if (PPBEC_ShowNativeCoolDown == nil) then
            PPBEC_ShowNativeCoolDown = false;
        end
        if (PPBEC_ShowOriginalTextTimer == nil) then
            PPBEC_ShowOriginalTextTimer = 1;
        end
        if (PPBEC_HandleWeaponBuff == nil) then
            PPBEC_HandleWeaponBuff = false;
        end
        if (PPBEC_DisplayCastableBuffs == nil) then
            PPBEC_DisplayCastableBuffs = false;
        end
        if (PPBEC_DisplayCurableDebuff == nil) then
            PPBEC_DisplayCurableDebuff = false;
        end
        if (PPBEC_ShowSecondUnder10m == nil) then
            PPBEC_ShowSecondUnder10m = false;
        end
        
        -- Call any code we need to activate them
        Perl_Player_UseBuffs(showbuffs);
        Perl_Player_Buff_Set_Scale();
        Perl_Player_Buff_Align();
    end
    
    PPB_Enhanced_Config[UnitName("player")] = {
        ["BuffPerLine"] = PPBEC_BuffPerLine,
        ["ShowNativeCoolDown"] = PPBEC_ShowNativeCoolDown,
        ["ShowOriginalTextTimer"] = PPBEC_ShowOriginalTextTimer,
        ["HandleWeaponBuff"] = PPBEC_HandleWeaponBuff,
        ["DisplayCastableBuffs"] = PPBEC_DisplayCastableBuffs,
        ["DisplayCurableDebuff"] = PPBEC_DisplayCurableDebuff,
        ["ShowSecondUnder10m"] = PPBEC_ShowSecondUnder10m,
    };
end

----------------------
-- myAddOns Support --
----------------------
function Perl_Player_Buff_myAddOns_Support()
    -- Register the addon in myAddOns
    if(myAddOnsFrame_Register) then
        local Perl_Player_Buff_myAddOns_Details = {
            name = "Perl_Player_Buff",
            version = Perl_Player_Buff_Version,
            releaseDate = "December 30, 2009",
            author = "Perl; Maintained by Global, Updated by Leliel",
            email = "mzhg@13fr.com",
            website = "http://wow.curse.com/downloads/wow-addons/details/perl-player-buff.aspx",
            category = MYADDONS_CATEGORY_OTHERS
        };
        Perl_Player_Buff_myAddOns_Help = {};
        Perl_Player_Buff_myAddOns_Help[1] = "/perl";
        myAddOnsFrame_Register(Perl_Player_Buff_myAddOns_Details, Perl_Player_Buff_myAddOns_Help);
    end
end